import gtk
import pygtk
import gobject
from desktop import DesktopParser
from os.path import expanduser, isfile,  join
from utils import ConfirmAction, get_icon, nullstring

# Constants for column mapping
(COLUMN_ICON,
 COLUMN_NAME,
 COLUMN_DESKTOP,
 COLUMN_PATH) = range(4)


class MainWindow(object):

    builder = gtk.Builder()

    def __init__(self, glade_file="application.glade"):
        # XXX: Check if the file exists.
        window_name = "window1"
        self.builder.add_from_file(glade_file)
        self.window = self.builder.get_object(window_name)

        # launcher Objects
        self.listview_launcher = self.builder.get_object('ListViewLauncher')
        self.LauncherModel = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING,
            gobject.TYPE_PYOBJECT, gobject.TYPE_STRING )
        render_pixbuf = gtk.CellRendererPixbuf()
        render_text = gtk.CellRendererText()
        col = gtk.TreeViewColumn()
        col.pack_start(render_pixbuf, expand=False)
        col.add_attribute(render_pixbuf, 'pixbuf', COLUMN_ICON)
        col.pack_start(render_text, expand=True)
        col.add_attribute(render_text, 'text', COLUMN_NAME)
        self.listview_launcher.append_column(col)
        self.listview_launcher.set_model(self.LauncherModel)

        # groups objects
        self.tvwGroups = self.builder.get_object('tvwGroups')
        self.GroupsModel = gtk.TreeStore(gobject.TYPE_STRING,
            gobject.TYPE_STRING )
        col = gtk.TreeViewColumn('Quicklist Groups',
            gtk.CellRendererText(),text=0)
        self.tvwGroups.append_column(col)

        cell_renderer = gtk.CellRendererText();
        cell_renderer.set_property('editable', True);
        col = gtk.TreeViewColumn('',cell_renderer, text=1);
        self.tvwGroups.append_column(col)

        self.tvwGroups.set_model(self.GroupsModel)

        # info Widgets
        self.imgIcon = self.builder.get_object('imgIcon')
        self.txtName = self.builder.get_object('txtName')
        self.txtComment = self.builder.get_object('txtComment')
        self.txtCommand = self.builder.get_object('txtCommand')
        self.chkTerminal=  self.builder.get_object('chkTerminal')

        # button widgets
        self.cmdIcon = self.builder.get_object('cmdIcon')
        self.cmdSave = self.builder.get_object('cmdSave')

        # set signals
        self.setup_signals()

    def setup_signals(self):
        # window close
        self.window.connect('delete-event',gtk.main_quit)
        self.builder.get_object('cmdClose').connect('clicked',gtk.main_quit)
        # launcher related Signals
        self.builder.get_object('cmdAdd').connect('clicked',
            self.add_new_launcher)
        self.builder.get_object('cmdRemove').connect('clicked',
            self.remove_from_launcher)
        self.cmdIcon.connect('clicked',self.load_icon_from_file)

    def show(self):
        self.window.show_all()

    def remove_from_launcher(self, widget):
        model, tree_iter  = self.get_selected_launcher_row()
        if tree_iter:
            dialog = ConfirmAction('Remove Launcher',
                'You are about to remove a Unity Launcher, are you sure?')
            if dialog.run():
                model.remove(tree_iter)
        else:
            ConfirmAction('Removing Launcher', 'No launcher selected',
                gtk.BUTTONS_OK ).run()

    def add_new_launcher(self, widget):
        from utils import InputBox
        dialog = InputBox('Name:', 'Inform the launcher name',
            title = 'Add a new launcher...',
            info = 'Type a launcher name to create a new launcher.')
        result,  text =  dialog.run()
        if result == gtk.RESPONSE_OK and len(text) > 0:
            desktop_parser = DesktopParser()
            desktop_parser.set('Name',  text)
            desktop_parser.set('Terminal', False)
            path = join(expanduser('~/.local/share/applications/'),
                '%s.desktop' % text )
            self.LauncherModel.append((get_icon('wft'), text, desktop_parser,
                path))
        pass

    def load_icon_from_file(self, widget):
        # FIX: maybe we could put the following code in it's own class file or
        # util.
        dialog = gtk.FileChooserDialog("Open..",
                               None,
                               gtk.FILE_CHOOSER_ACTION_OPEN, (
                                   gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                   gtk.STOCK_OPEN, gtk.RESPONSE_OK)
                               )
        dialog.set_default_response(gtk.RESPONSE_OK)

        # XXX: Here maybe we could make a list with all mimetypes and patterns
        # somewhere else and just iterate over them here.
        filter = gtk.FileFilter()
        filter.set_name("Images")
        filter.add_mime_type("image/png")
        filter.add_mime_type("image/jpeg")
        filter.add_mime_type("image/gif")
        filter.add_pattern("*.png")
        filter.add_pattern("*.jpg")
        filter.add_pattern("*.gif")
        filter.add_pattern("*.tif")
        filter.add_pattern("*.xpm")
        dialog.add_filter(filter)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            filename = dialog.get_filename()
            pixbuf = get_icon(filename)
            self.imgIcon.set_from_pixbuf(pixbuf)
            # TODO: add code to write changes here
            # the best way is to put an apply button?
            model, tree_iter  = self.get_selected_launcher_row()
            objDesktop = model[tree_iter][COLUMN_DESKTOP]
            model[tree_iter][COLUMN_ICON] = pixbuf
            objDesktop.set('Icon', filename)
        dialog.destroy()

    def get_selected_launcher_row(self):
        selection = self.listview_launcher.get_selection()
        return selection.get_selected()

    def listview_launcher_row_change(self, data=None):
        model, tree_iter  = self.get_selected_launcher_row()

        objDesktop = model[tree_iter][COLUMN_DESKTOP]

        # show properties
        self.imgIcon.set_from_pixbuf(model[tree_iter][COLUMN_ICON])

        self.txtName.set_text(nullstring(objDesktop.get('Name')))
        self.txtComment.get_buffer().set_text(
            nullstring(objDesktop.get('Comment')))
        self.txtCommand.set_text(nullstring(objDesktop.get('Exec')))
        self.chkTerminal.set_active(objDesktop.get('Terminal'))

        items = objDesktop.get('X-Ayatana-Desktop-Shortcuts')

        gModel = self.tvwGroups.get_model()
        gModel.clear()

        if items:
            for group in items:
                root = gModel.append(None,(group, ''))
                for key in ('Name', 'Exec', 'TargetEnvironment'):
                    opt = objDesktop.get(key, "%s Shortcut Group" % group)
                    if opt:
                        gModel.append(root,(key, opt))

