import gtk

from gobject import GError
from os.path import isfile


def get_icon(filename):
    image_size = 48
    image = gtk.Image()
    icon_theme = gtk.icon_theme_get_default()

    if isfile(filename):
        image.set_from_file(filename)
        icon = image.get_pixbuf()
        # resize the image to 48 pixels
        if icon:
            icon = icon.scale_simple(image_size, image_size,
                gtk.gdk.INTERP_BILINEAR)
    else:
        try:
            # to fix skype error loading a named icon like skype.png and no
            # skype
            icon = icon_theme.load_icon(filename.split('.')[0], image_size, 0)
        except:
            # returns a default icon, which unity uses when no icon is found.
            try:
                icon = icon_theme.load_icon('twf', image_size, 0)
            except GError:
                # icon not found, defaulting to the last fallback
                icon = icon_theme.load_icon('image-missing', image_size, 0)
    return icon


def nullstring(text):
    if text is None:
        text = ''
    return text


class ConfirmAction(gtk.MessageDialog):
    """A message dialog that asks a yes/no question."""

    def __init__(self,  title, description, buttons=gtk.BUTTONS_YES_NO):
        text = "<span size='xx-large'>%s</span>\n\n%s" % (title, description)
        super(ConfirmAction, self).__init__(
            None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            gtk.MESSAGE_WARNING, buttons)
        self.set_title(title)
        self.set_markup(text)

    def run(self, destroy = True):
        """Returns True if yes was clicked, False otherwise."""
        resp = super(ConfirmAction, self).run()
        if destroy:
            self.destroy()
        if resp == gtk.RESPONSE_YES:
            return True
        else:
            return False


class InputBox(gtk.MessageDialog):
    """XXX: add docs."""

    def __init__(self, label, message, title='', info=None):
        super(InputBox, self).__init__(None,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            gtk.MESSAGE_QUESTION,
            gtk.BUTTONS_OK_CANCEL,
            None)
        self.set_title(title)
        text = "<span size='xx-large'>%s</span>\n\n%s" % (message, info)
        self.text = gtk.Entry()
        # create a horizontal box to pack the entry and a label
        hbox = gtk.HBox()
        hbox.pack_start(gtk.Label(label), False, 5, 5)
        hbox.pack_end(self.text)
        self.set_markup(text)
        # add and show it
        self.vbox.pack_end(hbox, True, True, 0)
        self.show_all()

    def run(self, destroy = True):
        result = super(InputBox, self).run()
        text = self.text.get_text()
        if destroy:
            self.destroy()
        return result, text
